--*******************************************************
-- Demo: MSSQL-XML 
-- Author: Dr. Can Trker
--*******************************************************

use xmldb;

--*******************************************************
-- Native storage of XML documents
--*******************************************************
-- Register an XML schema
--*******************************************************

CREATE XML SCHEMA COLLECTION bookxsd AS
'<?xml version="1.0"?>
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
	<xsd:annotation><xsd:documentation xml:lang="en">Example "book", Dr. Can Tuerker Januar 2004</xsd:documentation></xsd:annotation>
	<xsd:element name="book" type="book"/>
	<xsd:complexType name="book">
		<xsd:sequence>
			<xsd:element name="author" type="xsd:string" maxOccurs="unbounded"/>
			<xsd:element name="title" type="xsd:string"/>
			<xsd:element name="subtitle" type="xsd:string"/>
			<xsd:element name="publisher">
				<xsd:complexType>
					<xsd:simpleContent>
						<xsd:extension base="xsd:string">
							<xsd:attribute name="city" type="xsd:string"/>
						</xsd:extension>
					</xsd:simpleContent>
				</xsd:complexType>
			</xsd:element>
			<xsd:element name="edition" type="xsd:int" minOccurs="0"/>
			<xsd:choice minOccurs="0">
				<xsd:element name="covertext" type="xsd:string"/>
				<xsd:element name="description" type="xsd:string"/>
			</xsd:choice>
			<xsd:element name="price" minOccurs="0">
				<xsd:complexType>
					<xsd:simpleContent>
						<xsd:extension base="xsd:decimal">
							<xsd:attribute name="currency" type="currency"/>
						</xsd:extension>
					</xsd:simpleContent>
				</xsd:complexType>
			</xsd:element>
		</xsd:sequence>
		<xsd:attribute name="isbn" type="xsd:string"/>
		<xsd:attribute name="year" type="xsd:gYear"/>
	</xsd:complexType>
	<xsd:simpleType name="currency">
		<xsd:restriction base="xsd:string">
			<xsd:enumeration value="CHF"/>
			<xsd:enumeration value="EUR"/>
			<xsd:enumeration value="USD"/>
		</xsd:restriction>
	</xsd:simpleType>
</xsd:schema>';

--*******************************************************
-- List all schema collections stored in the database
--*******************************************************
SELECT * FROM sys.xml_schema_collections;

--*******************************************************
-- Create table having a typed XML column
--*******************************************************

CREATE TABLE book (
  Id       INT PRIMARY KEY,
  content  XML (bookxsd)
);

--*******************************************************
-- Load XML documents into the database
--*******************************************************

INSERT INTO book 
SELECT 1, xCol
FROM (SELECT * FROM OPENROWSET (BULK 'C:\XMLDIR\book1.xml', SINGLE_BLOB) AS xCol) AS R(xCol);

INSERT INTO book 
SELECT 2, xCol
FROM (SELECT * FROM OPENROWSET (BULK 'C:\XMLDIR\book2.xml', SINGLE_BLOB) AS xCol) AS R(xCol);

INSERT INTO book 
SELECT 3, xCol
FROM (SELECT * FROM OPENROWSET (BULK 'C:\XMLDIR\book3.xml', SINGLE_BLOB) AS xCol) AS R(xCol);

INSERT INTO book 
SELECT 4, xCol
FROM (SELECT * FROM OPENROWSET (BULK 'C:\XMLDIR\book4.xml', SINGLE_BLOB) AS xCol) AS R(xCol);

SELECT * FROM book;

--*******************************************************
-- Create a primary index
--*******************************************************

CREATE PRIMARY XML INDEX idx_content ON book (content);

--*******************************************************
-- Create a path index
--*******************************************************

CREATE XML INDEX idx_content_path ON book (content) USING XML INDEX idx_content FOR PATH;

--*******************************************************
-- Create a propert index
--*******************************************************

CREATE XML INDEX idx_content_property ON book (content) USING XML INDEX idx_content FOR PROPERTY;

--*******************************************************
-- Create a value index
--*******************************************************

CREATE XML INDEX idx_content_value ON book (content) USING XML INDEX idx_content FOR VALUE;

--*******************************************************
-- Create a full-text index
--*******************************************************

CREATE UNIQUE INDEX idx_Key ON book (Id);

CREATE FULLTEXT CATALOG xmldbftcat AS DEFAULT;

CREATE FULLTEXT INDEX ON book (content) KEY INDEX idx_Key;

--*******************************************************
-- Example queries using the XML methods (and XML indexes)
--*******************************************************

SELECT content.query('<book id="{sql:column("id")}" year="{//@year}"><title>{data(//title)}</title><authors>{for $a in //author return <author>{data($a)}</author>}</authors></book>') AS book
FROM book
WHERE  content.exist('/book[author[1] = "Gunter Saake"]') = 1;

SELECT content.query('//author') AS author_element
FROM book
WHERE  content.exist('/book[author[1] = "Gunter Saake"]') = 1;

SELECT content.value('(//title)[1]', 'nvarchar(max)') AS booktitle, content.value('(//author)[1]', 'nvarchar(max)') AS firstauthor
FROM book;

SELECT id, content FROM book WHERE content.exist('/book[@year = xs:gYear("2003")]') = 1;

SELECT id, content FROM book WHERE content.exist('/book/description') = 1;

SELECT id, content FROM book WHERE content.exist('//title[contains(.,"XML")]') = 1;

--*******************************************************
-- Using the full-text index
--*******************************************************

SELECT id, content FROM book WHERE CONTAINS(content, '(XML AND Buch)');

SELECT id, content FROM book WHERE CONTAINS(content, '(XML OR Buch)');

SELECT id, content FROM book WHERE CONTAINS(content, '("XML Buch")');

SELECT id, content FROM book WHERE CONTAINS(content, '("XML Schema")');

SELECT id, content FROM book WHERE CONTAINS(content, '("XML")');

--*******************************************************
-- Update XML values
--*******************************************************

SELECT content.query('//author') AS authors
FROM book;

UPDATE book 
SET content.modify('insert <author>Unknown</author> before (//author)[1]');

SELECT content.query('//author') AS authors
FROM book;

UPDATE book 
SET content.modify('delete (//author)[1]');

SELECT content.query('//author') AS authors
FROM book;

SELECT content.query('//publisher') AS publisher
FROM book;

UPDATE book 
SET content.modify('replace value of (//publisher[. = "dpunkt"]/@city)[1] with "Zrich"');

SELECT content.query('//publisher') AS publisher
FROM book;
