--*******************************************************
-- Demo: Oracle-XML 
-- author: Dr. Can Trker
--*******************************************************

SET FEEDBACK 1;
--SET NUMWIDTH 50;
--SET TRIMSPOOL ON;
--SET TAB OFF;
--SET PAGESIZE 100;
SET LINESIZE 80;
SET serveroutput ON;
SET termout ON;
SET flush ON;
SET ECHO ON;
SET LONG 8000;

--*******************************************************
-- Register directories from where XML documents can be imported
--*******************************************************

CREATE DIRECTORY XMLDIR AS 'C:\XMLDIR';
GRANT READ ON DIRECTORY XMLDIR TO PUBLIC WITH GRANT OPTION;

--*******************************************************
-- Function for importing XML documents
--*******************************************************

CREATE FUNCTION getDocument(filename VARCHAR2) 
RETURN CLOB IS
  xclob CLOB;
BEGIN
  SELECT TO_CLOB(BFILENAME('XMLDIR', filename)) INTO xclob FROM DUAL;
  RETURN xclob;
END;
/

--*******************************************************
-- Object-relational storage of XML documents
-- 1) Register the XML-Schema
-- 2) Create schema-based XML table (object-relational storage by default)
--*******************************************************

BEGIN
  DBMS_XMLSCHEMA.deleteSchema('book.xsd', 4);
END;
/

BEGIN
  DBMS_XMLSCHEMA.registerSchema('book.xsd', getDocument('book.xsd'), false, false, false, false);
  DBMS_XMLSCHEMA.compileSchema('book.xsd');
END;
/

CREATE TABLE book OF XMLTYPE
XMLSCHEMA "book.xsd" 
ELEMENT "book";

-- Alternative syntax: 
-- CREATE TABLE book OF XMLTYPE
-- ELEMENT "book.xsd#book";

--*******************************************************
-- Meta-data about XML data in the database
--*******************************************************

DESC book;

COLUMN Schema     FORMAT a68; 
COLUMN Schema_URL FORMAT a10;

SELECT Schema, Schema_URL
FROM USER_XML_SCHEMAS;

COLUMN TABLE_NAME   FORMAT a20;
COLUMN STORAGE_TYPE FORMAT a20;
COLUMN XMLSCHEMA    FORMAT a20;

SELECT TABLE_NAME, STORAGE_TYPE, XMLSCHEMA
FROM USER_XML_TABLES;

--SELECT *
--FROM USER_XML_TAB_COLS;

--*******************************************************
-- Load XML documents into the database
--*******************************************************

INSERT INTO book VALUES (XMLTYPE(getDocument('book1.xml')));
INSERT INTO book VALUES (XMLTYPE(getDocument('book2.xml')));
INSERT INTO book VALUES (XMLTYPE(getDocument('book3.xml')));
INSERT INTO book VALUES (XMLTYPE(getDocument('book4.xml')));

--*******************************************************
-- Example queries using SQL/XML functions
--*******************************************************

COLUMN book FORMAT a78;
COLUMN booktitle FORMAT a45;
COLUMN isbn FORMAT a20;

SELECT XMLQUERY('<book Year="{$i//@year}"> 
                   <title>{$i//title/text()}</title>
                   <authors>{for $a in $i//author return <author>{$a/text()}</author>}</authors>  
                 </book>'                                                                                                                    
                 PASSING VALUE(b) AS "i" RETURNING CONTENT) AS book
FROM book b
WHERE XMLQUERY('$i/book[author[1] = "Gunter Saake"]' PASSING VALUE(b) AS "i" RETURNING CONTENT) IS NOT NULL;

SELECT t.*
FROM book b, XMLTABLE('$i' PASSING VALUE(b) AS "i" COLUMNS
	isbn VARCHAR(30) PATH '/book/@isbn', 
	booktitle VARCHAR(50) PATH '/book/title') AS t;

SELECT XMLQuery('$i//title/text()' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS booktitle
FROM book b
WHERE XMLEXISTS('$i//author[text() = "Gunter Saake"]' PASSING VALUE(b) AS "i");

SELECT XMLCAST(XMLQUERY('$i//book/title' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS CLOB) AS booktitle
FROM book b;

SELECT XMLCAST(XMLQUERY('$i//book/@year' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS CLOB) AS booktitle
FROM book b;

--*******************************************************
-- Set AUTOTRACE on EXPLAIN to get the query execution plan
--*******************************************************
SET AUTOTRACE ON EXPLAIN;

--*******************************************************
-- Create and use a functional index
--*******************************************************

SELECT XMLQUERY('<book title="{$i//title/text()}" year="{$i//@year}"/>' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS book
FROM book b
WHERE  XMLCAST(XMLQUERY('$i//@year' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS INTEGER) > 2000;

CREATE INDEX xmlfunktionalidx ON book b 
(XMLCAST(XMLQUERY('$i//@year' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS INT));

SELECT XMLQUERY('<book title="{$i//title/text()}" year="{$i//@year}"/>' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS book
FROM book b
WHERE  XMLCAST(XMLQUERY('$i//@year' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS INTEGER) > 2000;

--*******************************************************
-- Create and use a full-text index
--*******************************************************
-- Contains function requires a full-text index
--*******************************************************
-- If base data is updated, the index must be update explicitly 
-- using the following command:
-- ALTER INDEX <Name> REBUILD;
--*******************************************************

CREATE INDEX xmlfulltextidx ON book b 
(VALUE(b)) INDEXTYPE IS CTXSYS.CONTEXT;

SELECT XMLQUERY('$i//@isbn' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS isbn, SCORE(0)
FROM book b
WHERE CONTAINS(VALUE(b), 'Das Buch', 0) > 0
ORDER BY SCORE(0) DESC;

--*******************************************************
-- Switch off AUTOTRACE
--*******************************************************
SET AUTOTRACE OFF;

--*******************************************************
-- Get meta-data about XML indexes
--*******************************************************

COLUMN name         FORMAT a17;
COLUMN type         FORMAT a21;
COLUMN "type owner" FORMAT a10;
COLUMN "type name"  FORMAT a10;
COLUMN status       FORMAT a6;

SELECT index_name      name,
       index_type      type,
       ityp_owner      "type owner",
       ityp_name       "type name",
       domidx_opstatus status
FROM user_indexes
WHERE index_name LIKE 'XML%';

--*******************************************************
-- Update XML elements
--*******************************************************

COLUMN publisher FORMAT a40;
COLUMN publisherscity FORMAT a30;

CREATE TRIGGER bookschemacheck BEFORE INSERT OR UPDATE ON book FOR EACH ROW
DECLARE 
  newxml XMLType;
BEGIn
  newxml := :new.OBJECT_VALUE;
  XMLTYPE.schemavalidate(newxml);
END;
/

SELECT XMLQUERY('$i//publisher/text()' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS publisher, XMLQUERY('$i//@city' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS publisherscity FROM book b;

UPDATE book b
SET VALUE(b) = XMLQuery('copy $tmp := $i modify
                          ((for $p in $tmp//publisher[text()="dpunkt"]/@city
                            return replace value of node $p with $newCity),
                           (for $p in $tmp//publisher[text()="dpunkt"]
                            return replace value of node $p with $newName),
                            delete node $tmp//book[@isbn="3-89864-148-1"]/author[text()="Holger Meyer"])
                          return $tmp'
                          PASSING VALUE(b) AS "i", 'Zrich' AS "newCity", 'dpunkt-Verlag' AS "newName" RETURNING CONTENT);

SELECT XMLQUERY('$i//publisher/text()' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS publisher, XMLQUERY('$i//@city' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS publisherscity FROM book b;

CREATE VIEW XMLDBbook AS
SELECT VALUE(b) AS book
FROM book b
WHERE XMLEXISTS('$i//book[@isbn="3-89864-148-1"]' PASSING VALUE(b) AS "i");

SELECT * FROM XMLDBbook;

UPDATE book b
SET VALUE(b) = XMLQuery('copy $tmp := . modify
                          (delete node $tmp//author[text()="Holger Meyer"],
                           (for $a in $tmp//author[1]
                            return insert node <author>Holger Meier</author> after $a),
                           (for $b in $tmp//book
                            return insert node <author>Andreas Heuer</author> as first into $b))
                         return $tmp'
                         PASSING VALUE(b) RETURNING CONTENT);

SELECT * FROM XMLDBbook;

UPDATE book b
SET VALUE(b) = XMLQuery('copy $tmp := . modify
                          (for $b in $tmp//book
                           return insert node <newnode>test</newnode> into $b)
                         return $tmp'
                         PASSING VALUE(b) RETURNING CONTENT);

SELECT * FROM XMLDBbook;

--*******************************************************
-- Full-Text Index Rebuild
--*******************************************************

ALTER INDEX xmlfulltextidx REBUILD;

SELECT XMLQUERY('$i//publisher/text()' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS publisher,
       XMLQUERY('$i//title/text()' PASSING VALUE(b) AS "i" RETURNING CONTENT) AS booktitle
FROM book b;

--*******************************************************
-- Create XML view
--*******************************************************

CREATE VIEW Dpunktbook OF XMLTYPE WITH OBJECT ID DEFAULT AS 
SELECT *
FROM  book b
WHERE XMLEXISTS('$i//publisher[text()="dpunkt-Verlag"]' PASSING VALUE(b) AS "i");

SELECT * -- sys_nc_rowinfo$
FROM Dpunktbook;

EXIT;
/

